@@ -46,6 +46,7 @@ KSaneWidget::KSaneWidget(QWidget *parent)
     connect(d->m_ksaneCoreInterface, &KSaneCore::scanFinished, d, &KSaneWidgetPrivate::scanDone);
     connect(d->m_ksaneCoreInterface, &KSaneCore::userMessage, d, &KSaneWidgetPrivate::alertUser);
     connect(d->m_ksaneCoreInterface, &KSaneCore::scanProgress, d, &KSaneWidgetPrivate::updateProgress);
+    connect(d->m_ksaneCoreInterface, &KSaneCore::batchModeCountDown, d, &KSaneWidgetPrivate::updateCountDown);
     connect(d->m_ksaneCoreInterface, &KSaneCore::availableDevices, d, &KSaneWidgetPrivate::signalDevListUpdate);
     connect(d->m_ksaneCoreInterface, &KSaneCore::buttonPressed, this, &KSaneWidget::buttonPressed);
     connect(d->m_ksaneCoreInterface, &KSaneCore::openedDeviceInfoUpdated, this, &KSaneWidget::openedDeviceInfoUpdated);
@@ -59,15 +60,16 @@ KSaneWidget::KSaneWidget(QWidget *parent)
     d->m_warmingUp = new QLabel;
     d->m_warmingUp->setText(i18n("Waiting for the scan to start."));
     d->m_warmingUp->setAlignment(Qt::AlignCenter);
-    d->m_warmingUp->setAutoFillBackground(true);
-    d->m_warmingUp->setBackgroundRole(QPalette::Highlight);
-    //d->m_warmingUp->setForegroundRole(QPalette::HighlightedText);
     d->m_warmingUp->hide();
 
+    d->m_countDown = new QLabel;
+    d->m_countDown->setAlignment(Qt::AlignCenter);
+    d->m_countDown->hide();
+
     d->m_progressBar = new QProgressBar;
     d->m_progressBar->setMaximum(100);
 
-    d->m_cancelBtn   = new QPushButton;
+    d->m_cancelBtn = new QPushButton;
     d->m_cancelBtn->setIcon(QIcon::fromTheme(QStringLiteral("process-stop")));
     d->m_cancelBtn->setToolTip(i18n("Cancel current scan operation"));
     connect(d->m_cancelBtn, &QPushButton::clicked, this, &KSaneWidget::scanCancel);
@@ -76,6 +78,8 @@ KSaneWidget::KSaneWidget(QWidget *parent)
     QHBoxLayout *progress_lay = new QHBoxLayout(d->m_activityFrame);
     progress_lay->setContentsMargins(0, 0, 0, 0);
     progress_lay->addWidget(d->m_progressBar, 100);
+    progress_lay->addWidget(d->m_warmingUp, 100);
+    progress_lay->addWidget(d->m_countDown, 100);
     progress_lay->addWidget(d->m_cancelBtn, 0);
     d->m_activityFrame->hide();
 
@@ -126,6 +130,7 @@ KSaneWidget::KSaneWidget(QWidget *parent)
     d->m_scanBtn->setToolTip(i18n("Scan Final Image (%1)", scanShortcut->key().toString()));
     d->m_scanBtn->setText(i18nc("Final scan button text", "Scan"));
     d->m_scanBtn->setFocus(Qt::OtherFocusReason);
+    setFocusProxy(d->m_scanBtn);
     connect(d->m_scanBtn, &QToolButton::clicked, d, &KSaneWidgetPrivate::startFinalScan);
 
     d->m_btnFrame = new QWidget;
@@ -145,18 +150,15 @@ KSaneWidget::KSaneWidget(QWidget *parent)
     if (d->m_activityFrame->sizeHint().height() > minHeight) {
         minHeight = d->m_activityFrame->sizeHint().height();
     }
-    if (d->m_warmingUp->sizeHint().height() > minHeight) {
-        minHeight = d->m_warmingUp->sizeHint().height();
-    }
     d->m_btnFrame->setMinimumHeight(minHeight);
     d->m_activityFrame->setMinimumHeight(minHeight);
     d->m_warmingUp->setMinimumHeight(minHeight);
+    d->m_countDown->setMinimumHeight(minHeight);
 
     d->m_previewFrame = new QWidget;
     QVBoxLayout *preview_layout = new QVBoxLayout(d->m_previewFrame);
     preview_layout->setContentsMargins(0, 0, 0, 0);
     preview_layout->addWidget(d->m_previewViewer, 100);
-    preview_layout->addWidget(d->m_warmingUp, 0);
     preview_layout->addWidget(d->m_activityFrame, 0);
     preview_layout->addWidget(d->m_btnFrame, 0);
 
@@ -169,6 +171,12 @@ KSaneWidget::KSaneWidget(QWidget *parent)
     d->m_basicScrollA->setFrameShape(QFrame::NoFrame);
     d->m_optsTabWidget->addTab(d->m_basicScrollA, i18n("Basic Options"));
 
+    // Add the advanced options tab
+    d->m_advancedScrollA = new QScrollArea();
+    d->m_advancedScrollA->setWidgetResizable(true);
+    d->m_advancedScrollA->setFrameShape(QFrame::NoFrame);
+    d->m_optsTabWidget->addTab(d->m_advancedScrollA, i18n("Advanced Options"));
+
     // Add the other options tab
     d->m_otherScrollA = new QScrollArea;
     d->m_otherScrollA->setWidgetResizable(true);
@@ -261,9 +269,7 @@ QString KSaneWidget::selectDevice(QWidget *parent)
     connect(d->m_ksaneCoreInterface, &KSaneCore::availableDevices, sel, &KSaneDeviceDialog::updateDevicesList);
     connect(sel, &KSaneDeviceDialog::requestReloadList, d->m_ksaneCoreInterface, &KSaneCore::reloadDevicesList);
     
-    // set default scanner - perhaps application using libksane should remember that
-    // 2014-01-21: gm: +1
-    // sel.setDefault(prev_backend);
+    d->m_ksaneCoreInterface->reloadDevicesList();
 
     if (sel->exec() == QDialog::Accepted) {
         selected_name = sel->getSelectedName();
@@ -368,7 +374,7 @@ bool KSaneWidget::closeDevice()
     if (!result) {
         return false;
     }
- 
+    d->clearDeviceOptions();
     // disable the interface until a new device is opened.
     d->m_optsTabWidget->setDisabled(true);
     d->m_previewViewer->setDisabled(true);
